#!/bin/sh
#\
exec wish "$0"

wm title . Euhedral


#
# Define Procedures
#
##########################################
# Send command and command parameter(s) to the program. Log output.
proc OneCommand {command} {
        global f 
        puts $f $command
        flush $f
        update
        LogUntilPrompt
}

proc LastCommand {} {
        global f 
        puts $f "exit"
        flush $f
        update
        exit
#        LogUntilPrompt
}
##########################################
# Read and log one line of output from the program
proc Log1 {} {
    global log f line DoEcho
    
#    gets $f line
#    if [eof $f] {
#      puts stdout "End-Of-File from euhedral. Terminating..."
#	exit
#    } 
    update
    set nb [ gets $f line ]
    while { $nb == -1 } {
#	puts stdout "no input"
	if [eof $f] {
	    set line "End-Of-File from euhedral. Terminating..."
	    puts stdout $line
	    WarningMessage $line
	    update
	    after 5000
	    exit
	} else {
#	    puts stdout "Fblocked = [fblocked $f]"
	}
	update
	after 1000
	set nb [ gets $f line ]
    }
#    puts stdout "Log1: $line"
    if { [string equal -length 8 $line "WARNING:" ] } {
	puts stdout "$line"
    }
    if { $DoEcho == 1 } {
	if { ! [string equal -length 7 $line "IGNORE:" ] } {
	    $log insert end $line\n
	    $log see end
	    update
	}
    }
}
##########################################
# Read and log lines of output from the program until a prompt
proc LogUntilPrompt {} {
	global log line
	Log1
        while { ! [string equal -length 7 $line "PROMPT:" ] } {
	  Log1
        }
        set line [ string range $line 8 end ]
}

##########################################
# Read and log lines of output from the program until a prompt
proc GetValue {command varname value} {
    upvar $value localv
    global log f line DoEcho

    set DoEcho 0
    
    puts $f "$command"
    flush $f
    Log1
    set ls [string length $varname]
    while { ! [string equal -length 6 $line "PROMPT:" ] } {
	if [string equal -length $ls $line $varname ] {
	    set localv [string trim [ string range $line $ls end ] ]
	}
	Log1
    }
    set line [ string range $line 8 end ]
    set DoEcho 1
}

##########################################
# Read and log lines of output from the program until a prompt
proc GetStatus {} {
	global log f line DoEcho
        global options

        puts $f "status"
        flush $f
        set DoEcho 0    
	Log1
        while { ! [string equal -length 6 $line "PROMPT:" ] } {
	    set line [ string trim $line ]
	    set pos [ string first " " $line ]
            set key [ string range $line 0 $pos]
	    set restline [ string trim [ string range $line $pos end] ]

	    if { [string equal $key "angdistmax " ] } {
		scan $restline %f options(angdistmax)
	    } elseif { [string equal $key "angmax " ] } {
		scan $restline %f options(angmax)
	    } elseif { [string equal $key "angmin " ] } {
		scan $restline %f options(angmin)
	    } elseif { [string equal $key "centric " ] } {
		scan $restline %s options(centric)
	    } elseif { [string equal $key "correction " ] } {
		scan $restline %s options(correction)
	    } elseif { [string equal $key "delphi " ] } {
		scan $restline %f options(delphi)
	    } elseif { [string equal $key "extreme " ] } {
		scan $restline %f options(extreme)
	    } elseif { [string equal $key "highindex " ] } {
		scan $restline %d options(highindex)
	    } elseif { [string equal $key "nhklmin " ] } {
		scan $restline %d options(nhklmin)
	    } elseif { [string equal $key "initangle " ] } {
		scan $restline %f options(initangle)
	    } elseif { [string equal $key "initdistance " ] } {
		scan $restline %f options(initdistance)
	    } elseif { [string equal $key "intmin " ] } {
		scan $restline %f options(intmin)
	    } elseif { [string equal $key "mode " ] } {
		scan $restline %s options(mode)
	    } elseif { [string equal $key "pg " ] } {
		scan $restline %s options(pg)
	    } elseif { [string equal $key "phistart " ] } {
		scan $restline %f options(phistart)
	    } elseif { [string equal $key "phistop " ] } {
		scan $restline %f options(phistop)
	    } elseif { [string equal $key "plotaxes " ] } {
		scan $restline %s options(plotaxes)
	    } elseif { [string equal $key "plotcorner " ] } {
		scan $restline %s options(plotcorner)
	    } elseif { [string equal $key "plotlab " ] } {
		scan $restline %s options(plotlab)
	    } elseif { [string equal $key "thetamin " ] } {
		scan $restline %f options(thetamin)
	    } elseif { [string equal $key "tol " ] } {
		scan $restline %f options(tol)
	    } elseif { [string equal $key "vertical " ] } {
		scan $restline {%s} options(vertical)
	    } elseif { [string equal $key "weight " ] } {
		scan $restline {%f %f} options(weight1) options(weight2)
	    }
	    Log1
        }
        set line [ string range $line 8 end ]
        set DoEcho 1
}

proc InfoMessage {s} {
    OneMessage "Info: $s"
}

proc WaitMessage {s} {
    OneMessage "Please wait: $s"
}
proc WarningMessage {s} {
    OneMessage "WARNING: $s"
}

proc OneMessage {s} {
    CleanMessage
    update idletasks
    button .m -bg yellow -fg black -activebackground yellow -height 2 -text $s
    pack .m -expand true -fill x -in .messageFrame
}

proc CleanMessage {} {
    destroy .m
}

proc PrintOptions {} {
    global log options
    $log insert end "Current Option Values:\n"
    $log insert end "angdistmax $options(angdistmax) \n"
    $log insert end "angmax $options(angmax) \n"
    $log insert end "angmin $options(angmin) \n"
    $log insert end "centric $options(centric) \n"
    $log insert end "correction $options(correction) \n"
    $log insert end "delphi $options(delphi) \n"
    $log insert end "extreme $options(extreme) \n"
    $log insert end "highindex $options(highindex) \n"
    $log insert end "initangle $options(initangle) \n"
    $log insert end "initdistance $options(initdistance) \n"
    $log insert end "intmin $options(intmin) \n"
    $log insert end "mode $options(mode) \n"
    $log insert end "nhklmin $options(nhklmin) \n"
    $log insert end "pg $options(pg) \n"
    $log insert end "phistart $options(phistart) \n"
    $log insert end "phistop $options(phistop) \n"
    $log insert end "plotaxes $options(plotaxes) \n"
    $log insert end "plotcorner $options(plotcorner) \n"
    $log insert end "plotlab $options(plotlab) \n"
    $log insert end "thetamin $options(thetamin) \n"
    $log insert end "tol $options(tol) \n"
    $log insert end "vertical $options(vertical)\n"
    $log insert end "weight $options(weight1) $options(weight2) \n"
    $log insert end "============================\n"
    $log see end

}

##########################################
# Handle abort button and messages
proc RaiseAbort {lab} {
    WaitMessage "$lab"
#    button .m -bg yellow -height 2 -text 
    button .abort -background red -activebackground red -text "Abort" -height 1 -command {
	exec kill -INT $euhedralpid
#	puts stdout "send SIGINT to $euhedralpid"
        set abortfile [open "euhedral.abort" "w" "0600"]
        close $abortfile
#	puts stdout "created euhedral.abort"
	update
	.m configure -text "Abort in progress"
	update
    }
    pack .m -expand true -fill x -in .messageFrame
    pack .abort -expand true  -in .messageFrame
}

proc RemoveAbort {} {
    destroy .abort
    CleanMessage
}

##########################################
# Choose ins file
proc selectIns {} {
    global filename log line options
    
    set types {
	{{PLATON-ins-file}       {.ins}        }
	{{PLATON-res-file}       {.res}        }
	{{All Files}        *             }
    }
    
    set filename(ins) [tk_getOpenFile -filetypes $types]
    set filename(ins) [file tail $filename(ins)]
    if {$filename(ins) != ""} {
	if { $options(pg) == "?" } {
	    guessPg
	    $log insert end "pg guessed: $options(pg) \n"
	    $log see end

	}
	WaitMessage "reading ins"
	OneCommand "readins $filename(ins)"
	InfoMessage "select pointgroup"
	selectPg
	tkwait window .pointgroups 
	CleanMessage
	if { $options(hklread) == 1 } {
	    InfoMessage "next: select hkl file"
	}
    }
}                                                                        

##########################################
# Choose hkl file
proc selectHKL { } {
    global f log line options
    
    set types {
	{{PLATON-hkl-file}       {.hkl}        }
	{{All Files}        *             }
    }
    
    set filename(hkl) [tk_getOpenFile -filetypes $types]
    set filename(hkl) [file tail $filename(hkl)]
    if {$filename(hkl) != ""} {
	WaitMessage "reading hkl data"
#	update idletasks
	OneCommand "readhkl $filename(hkl)"
	CleanMessage
	set options(hklread) 0
	if { $options(filterrun) == 1 } {
	    InfoMessage "next: run filter"
	}
    }
}                                                                        

##########################################
# Guess point group
proc guessPg {} {
    global filename options
    set infile [open $filename(ins) "r"]

    while { ! [eof $infile]} {
	gets $infile line

        if {[string equal -length 4 $line "CELL" ] } {
	    set fmt {%f %f %f %f %f %f %f}
	    scan [ string range $line 4 end] $fmt lambda options(cella) \
		    options(cellb) options(cellc) options(cellalpha) \
		    options(cellbeta) options(cellgamma)
	}
    }
    close $infile
    if {$options(cellalpha) == 90.0 && $options(cellbeta) == 90.0 && \
	    $options(cellgamma) == 90.0 && $options(cella) == $options(cellb) && \
	    $options(cella) == $options(cellc)} {
	set options(pg) "m3m"
    } elseif {$options(cellalpha) == 90.0 && $options(cellbeta) == 90.0 && \
	    $options(cellgamma) == 120.0 && $options(cella) == $options(cellb) } {
	set options(pg) "6/mmm"
    } elseif {$options(cellalpha) == $options(cellbeta) && $options(cellalpha) \
	    == $options(cellgamma) && $options(cellalpha) != 90.0 && \
	    $options(cella) == $options(cellb) && $options(cella) == \
	    $options(cellc)} {
	set options(pg) "R-3m"
    } elseif {$options(cellalpha) == 90.0 && $options(cellbeta) == 90.0 && \
	    $options(cellgamma) == 90.0 && $options(cella) == $options(cellb) } {
	set options(pg) "4/mmm"
    } elseif {$options(cellalpha) == 90.0 && $options(cellbeta) == 90.0 && \
	    $options(cellgamma) == 90.0 } {
	set options(pg) "mmm"
    } elseif {$options(cellalpha) == 90.0 && $options(cellgamma) == 90.0 } {
	set options(pg) "2/m"
    } else {set options(pg) "-1"}
}

##########################################
# Select point group
proc selectPg {} {
global f log line options
toplevel .pointgroups
frame .pointgroups.top -borderwidth 10
pack .pointgroups.top  -side top -fill x
frame .pointgroups.bottom -borderwidth 10
pack .pointgroups.bottom  -side top -fill x

#triclinic
radiobutton .pointgroups.1 -text "1"  -variable options(pg) -value "1"
radiobutton .pointgroups.2 -text "-1" -variable options(pg) -value "-1"
#monoclinic
radiobutton .pointgroups.3 -text "2"   -variable options(pg) -value "2"
radiobutton .pointgroups.4 -text "m"   -variable options(pg) -value "m"
radiobutton .pointgroups.5 -text "2/m" -variable options(pg) -value "2/m"
#orthorhombic
radiobutton .pointgroups.6 -text "222" -variable options(pg) -value "222"
radiobutton .pointgroups.7 -text "mm2" -variable options(pg) -value "mm2"
radiobutton .pointgroups.8 -text "mmm" -variable options(pg) -value "mmm"
#tetragonal
radiobutton .pointgroups.9 -text "4"      -variable options(pg) -value "4"
radiobutton .pointgroups.10 -text "-4"    -variable options(pg) -value "-4"
radiobutton .pointgroups.11 -text "4/m"   -variable options(pg) -value "4/m"
radiobutton .pointgroups.12 -text "422"   -variable options(pg) -value "422"
radiobutton .pointgroups.13 -text "4mm"   -variable options(pg) -value "4mm"
radiobutton .pointgroups.14 -text "42m"   -variable options(pg) -value "42m"
radiobutton .pointgroups.15 -text "4m2"   -variable options(pg) -value "4m2"
radiobutton .pointgroups.16 -text "4/mmm" -variable options(pg) -value "4/mmm"
#trigonal
radiobutton .pointgroups.17 -text "3"    -variable options(pg) -value "3"
radiobutton .pointgroups.18 -text "-3"   -variable options(pg) -value "-3"
radiobutton .pointgroups.19 -text "312"  -variable options(pg) -value "312"
radiobutton .pointgroups.20 -text "321"  -variable options(pg) -value "321"
radiobutton .pointgroups.21 -text "31m"  -variable options(pg) -value "31m"
radiobutton .pointgroups.22 -text "3m1"  -variable options(pg) -value "3m1"
radiobutton .pointgroups.23 -text "-31m" -variable options(pg) -value "-31m"
radiobutton .pointgroups.24 -text "-3m1" -variable options(pg) -value "-3m1"
#hexagonal
radiobutton .pointgroups.25 -text "6"     -variable options(pg) -value "6"
radiobutton .pointgroups.26 -text "-6"    -variable options(pg) -value "-6"
radiobutton .pointgroups.27 -text "6/m"   -variable options(pg) -value "6/m"
radiobutton .pointgroups.28 -text "622"   -variable options(pg) -value "622"
radiobutton .pointgroups.29 -text "6mm"   -variable options(pg) -value "6mm"
radiobutton .pointgroups.30 -text "-6m2"  -variable options(pg) -value "-6m2"
radiobutton .pointgroups.31 -text "-62m"  -variable options(pg) -value "-62m"
radiobutton .pointgroups.32 -text "6/mmm" -variable options(pg) -value "6/mmm"
#rhombohedral
radiobutton .pointgroups.33 -text "R3"   -variable options(pg) -value "R3"
radiobutton .pointgroups.34 -text "R-3"  -variable options(pg) -value "R-3"
radiobutton .pointgroups.35 -text "R32"  -variable options(pg) -value "R32"
radiobutton .pointgroups.36 -text "R3m"  -variable options(pg) -value "R3m"
radiobutton .pointgroups.37 -text "R-3m" -variable options(pg) -value "R-3m"
#cubic
radiobutton .pointgroups.38 -text "23"   -variable options(pg) -value "23"
radiobutton .pointgroups.39 -text "m3"   -variable options(pg) -value "m3"
radiobutton .pointgroups.40 -text "432"  -variable options(pg) -value "432"
radiobutton .pointgroups.41 -text "-43m" -variable options(pg) -value "-43m"
radiobutton .pointgroups.42 -text "m3m"  -variable options(pg) -value "m3m"

grid .pointgroups.1   -in .pointgroups.top -row 0 -column 0 -sticky w
grid .pointgroups.2   -in .pointgroups.top -row 1 -column 0 -sticky w

grid .pointgroups.3   -in .pointgroups.top -row 0 -column 1 -sticky w
grid .pointgroups.4   -in .pointgroups.top -row 1 -column 1 -sticky w
grid .pointgroups.5   -in .pointgroups.top -row 2 -column 1 -sticky w

grid .pointgroups.6   -in .pointgroups.top -row 0 -column 2 -sticky w
grid .pointgroups.7   -in .pointgroups.top -row 1 -column 2 -sticky w
grid .pointgroups.8   -in .pointgroups.top -row 2 -column 2 -sticky w

grid .pointgroups.9    -in .pointgroups.top -row 0 -column 3 -sticky w
grid .pointgroups.10   -in .pointgroups.top -row 1 -column 3 -sticky w
grid .pointgroups.11   -in .pointgroups.top -row 2 -column 3 -sticky w
grid .pointgroups.12   -in .pointgroups.top -row 3 -column 3 -sticky w
grid .pointgroups.13   -in .pointgroups.top -row 4 -column 3 -sticky w
grid .pointgroups.14   -in .pointgroups.top -row 5 -column 3 -sticky w
grid .pointgroups.15   -in .pointgroups.top -row 6 -column 3 -sticky w
grid .pointgroups.16   -in .pointgroups.top -row 7 -column 3 -sticky w

grid .pointgroups.17   -in .pointgroups.top -row 0 -column 4 -sticky w
grid .pointgroups.18   -in .pointgroups.top -row 1 -column 4 -sticky w
grid .pointgroups.19   -in .pointgroups.top -row 2 -column 4 -sticky w
grid .pointgroups.20   -in .pointgroups.top -row 3 -column 4 -sticky w
grid .pointgroups.21   -in .pointgroups.top -row 4 -column 4 -sticky w
grid .pointgroups.22   -in .pointgroups.top -row 5 -column 4 -sticky w
grid .pointgroups.23   -in .pointgroups.top -row 6 -column 4 -sticky w
grid .pointgroups.24   -in .pointgroups.top -row 7 -column 4 -sticky w

grid .pointgroups.25   -in .pointgroups.top -row 0 -column 5 -sticky w
grid .pointgroups.26   -in .pointgroups.top -row 1 -column 5 -sticky w
grid .pointgroups.27   -in .pointgroups.top -row 2 -column 5 -sticky w
grid .pointgroups.28   -in .pointgroups.top -row 3 -column 5 -sticky w
grid .pointgroups.29   -in .pointgroups.top -row 4 -column 5 -sticky w
grid .pointgroups.30   -in .pointgroups.top -row 5 -column 5 -sticky w
grid .pointgroups.31   -in .pointgroups.top -row 6 -column 5 -sticky w
grid .pointgroups.32   -in .pointgroups.top -row 7 -column 5 -sticky w

grid .pointgroups.33   -in .pointgroups.top -row 0 -column 6 -sticky w
grid .pointgroups.34   -in .pointgroups.top -row 1 -column 6 -sticky w
grid .pointgroups.35   -in .pointgroups.top -row 2 -column 6 -sticky w
grid .pointgroups.36   -in .pointgroups.top -row 3 -column 6 -sticky w
grid .pointgroups.37   -in .pointgroups.top -row 4 -column 6 -sticky w

grid .pointgroups.38   -in .pointgroups.top -row 0 -column 7 -sticky w
grid .pointgroups.39   -in .pointgroups.top -row 1 -column 7 -sticky w
grid .pointgroups.40   -in .pointgroups.top -row 2 -column 7 -sticky w
grid .pointgroups.41   -in .pointgroups.top -row 3 -column 7 -sticky w
grid .pointgroups.42   -in .pointgroups.top -row 4 -column 7 -sticky w

button .pointgroups.ok -text "Select point group" -command {
destroy .pointgroups 
OneCommand "pg $options(pg)"
}
pack .pointgroups.ok -in .pointgroups.bottom -side top -fill x

}

##########################################
# Filter
proc filter {} {
    global options
    if { $options(hklread) == 0 } {
	CleanMessage
	set options(filterrun) 0
	toplevel .filter
	frame .filter.top -borderwidth 10
	pack .filter.top  -side top -fill x
	frame .filter.middle -borderwidth 10
	pack .filter.middle  -side top -fill x
	frame .filter.bottom -borderwidth 10
	pack .filter.bottom  -side top -fill x
	
	entry .filter.nhklmin -width 10 -relief sunken -textvariable options(nhklmin)
	label .filter.1 -text "Minimal Redundancy"
	entry .filter.thetamin -width 10 -relief sunken -textvariable options(thetamin)
	label .filter.2 -text "Minimal Theta"
	entry .filter.intmin -width 10 -relief sunken -textvariable options(intmin)
	label .filter.3 -text "Minimal Intensity"
	entry .filter.angdistmax -width 10 -relief sunken -textvariable \
		options(angdistmax)
	label .filter.4 -text "Angular Distribution"
	
	grid .filter.nhklmin -in .filter.top    -row 0 -column 0 -sticky w
	grid .filter.1 -in .filter.top          -row 0 -column 1 -sticky w
	grid .filter.thetamin -in .filter.top   -row 1 -column 0 -sticky w
	grid .filter.2 -in .filter.top          -row 1 -column 1 -sticky w
	grid .filter.intmin -in .filter.top     -row 2 -column 0 -sticky w
	grid .filter.3 -in .filter.top          -row 2 -column 1 -sticky w
	grid .filter.angdistmax -in .filter.top -row 3 -column 0 -sticky w
	grid .filter.4 -in .filter.top          -row 3 -column 1 -sticky w
	
	radiobutton .filter.stereographic -text "stereographic" \
		-variable options(mode) -value "stereographic"
	radiobutton .filter.cone -text "cone" \
		-variable options(mode) -value "cone"
	radiobutton .filter.z -text "z-projection" \
		-variable options(mode) -value "z"
	radiobutton .filter.ajmd -text "ajmd-projection" \
		-variable options(mode) -value "ajmd"
	
	grid .filter.stereographic -in .filter.middle -row 0 -column 0 -sticky w 
	grid .filter.cone -in .filter.middle -row 0 -column 1 -sticky w 
	grid .filter.z -in .filter.middle -row 0 -column 2 -sticky w
	grid .filter.ajmd -in .filter.middle -row 0 -column 3 -sticky w
	
	
	button .filter.filter -text "Run Filter" -command {
	    OneCommand "mode $options(mode)"
	    OneCommand "nhklmin $options(nhklmin)"
	    OneCommand "thetamin $options(thetamin)"
	    OneCommand "intmin $options(intmin)"
	    OneCommand "angdistmax $options(angdistmax)"
	    OneCommand filter
	}
	button .filter.keep -text "Keep results" -command {
	    OneCommand "writehkl"
	    destroy .filter 
	}
	button .filter.discharge -text "Cancel - Dismiss results" -command {
	    destroy .filter
	}
	pack .filter.filter -in .filter.bottom -side left -fill x
	pack .filter.keep -in .filter.bottom -side left -fill x
	pack .filter.discharge -in .filter.bottom -side left -fill x
    } else {
	WarningMessage "first select hkl file"	
    }
}

##########################################
# Distance refinement
proc invokeDistance {} {
global options
global refcommand

set refcommand "distance"
toplevel .mylevel
wm title .mylevel "Distance Refinement"

frame .mylevel.top -borderwidth 10  
frame .mylevel.bottom -borderwidth 10

pack .mylevel.top  -side top -fill x
pack .mylevel.bottom  -side top -fill x


label .mylevel.centriclabel  -text "Crystal Shape:"
radiobutton .mylevel.centricOn -text "Centric" -variable \
    options(centric) -value "on"
radiobutton .mylevel.centricOff -text "Noncentric" -variable \
    options(centric) -value "off"

label .mylevel.refinelabel  -text "Refinement Mode:"
radiobutton .mylevel.amoeba -text "Simplex (simultaneously)" -variable \
    refcommand -value "amoeba"
radiobutton .mylevel.brent -text "Parabolic (separately)" -variable \
    refcommand -value "distance"

label .mylevel.textExtreme   -text "Extreme:"
scale .mylevel.extreme       -from {0.1} -to {1.0} -orient horizontal -length 200 \
                     -digits 2 -resolution {0.1} -variable options(extreme)
.mylevel.extreme set $options(extreme)

label .mylevel.textTolerance -text "Tolerance:"
scale .mylevel.tolerance     -from {0.01} -to {0.1} -orient horizontal -length 200 \
                     -digits 2 -resolution {0.01} -variable options(tol)
.mylevel.tolerance set $options(tol)

grid .mylevel.centriclabel  -in .mylevel.top -row 1 -column 0 -sticky w
grid .mylevel.centricOn     -in .mylevel.top -row 1 -column 1 -sticky w
grid .mylevel.centricOff    -in .mylevel.top -row 1 -column 2 -sticky w
grid .mylevel.refinelabel   -in .mylevel.top -row 2 -column 0 -sticky w
grid .mylevel.brent         -in .mylevel.top -row 2 -column 1 -sticky w
grid .mylevel.amoeba        -in .mylevel.top -row 2 -column 2 -sticky w
grid .mylevel.textExtreme   -in .mylevel.top -row 3 -column 0 -sticky w
grid .mylevel.textTolerance -in .mylevel.top -row 4 -column 0 -sticky w
grid .mylevel.extreme       -in .mylevel.top -row 3 -column 1 -sticky w
grid .mylevel.tolerance     -in .mylevel.top -row 4 -column 1 -sticky w


button .mylevel.refine -text "Refine Distance"  -command {
    destroy .mylevel
    RaiseAbort "Refining Distance"
    OneCommand "extreme $options(extreme)"
    OneCommand "centric $options(centric)"
    OneCommand "tol $options(tol)"
    OneCommand "$refcommand"
    RemoveAbort 
}

pack .mylevel.refine -in .mylevel.bottom -side left -fill x
button .mylevel.cancel -text Cancel -command { destroy .mylevel }
pack .mylevel.cancel -in .mylevel.bottom -side left -fill x

wm geometry .mylevel +100+300
}

##########################################
# Volume refinement
proc invokeVolume {} {
global options

toplevel .mylevel
wm title .mylevel "Volume Refinement"
frame .mylevel.top -borderwidth 10
pack .mylevel.top  -side top -fill x
frame .mylevel.bottom -borderwidth 10
pack .mylevel.bottom  -side top -fill x

label .mylevel.textExtreme   -text Extreme
label .mylevel.textTolerance -text Tolerance

scale .mylevel.extreme       -from {0.1} -to {1.0} -orient horizontal -length 200 \
                     -digits 2 -resolution {0.1} -variable options(extreme)
scale .mylevel.tolerance     -from {0.01} -to {0.1} -orient horizontal -length 200 \
                     -digits 2 -resolution {0.01} -variable options(tol)

.mylevel.extreme set $options(extreme)
.mylevel.tolerance set $options(tol)


grid .mylevel.textExtreme   -in .mylevel.top -row 0 -column 0 -sticky sw
grid .mylevel.textTolerance -in .mylevel.top -row 1 -column 0 -sticky sw
grid .mylevel.extreme       -in .mylevel.top -row 0 -column 1 -sticky we
grid .mylevel.tolerance     -in .mylevel.top -row 1 -column 1 -sticky we


button .mylevel.refine -text "Refine Volume"  -command {
    destroy .mylevel
    RaiseAbort "Refining Volume"
    OneCommand "extreme $options(extreme)"
    OneCommand "tol $options(tol)"
    OneCommand "volume"
    RemoveAbort
}


pack .mylevel.refine -in .mylevel.bottom -side left -fill x
button .mylevel.cancel -text Cancel -command { destroy .mylevel }
pack .mylevel.cancel -in .mylevel.bottom -side left -fill x

wm geometry .mylevel +100+300

}

##########################################
# Tilt refinement
proc invokeTilt {} {
    global options
    global facearray

    toplevel .mylevel
    wm title .mylevel "Tilt Refinement"
    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x

    entry .mylevel.highindex -width 10 -relief sunken -textvariable options(highindex)
    label .mylevel.hi -text "HighIndex"
    entry .mylevel.angmax -width 10 -relief sunken -textvariable options(angmax)
    label .mylevel.ma -text "Maximum Angle"

    grid .mylevel.highindex -in .mylevel.top    -row 0 -column 0 -sticky w
    grid .mylevel.hi -in .mylevel.top            -row 0 -column 1 -sticky w
    grid .mylevel.angmax -in .mylevel.top       -row 1 -column 0 -sticky w
    grid .mylevel.ma -in .mylevel.top            -row 1 -column 1 -sticky w

    GetValue "gui faces" "faces" nfaces
    label .mylevel.label2 -text "face number(s)"
    pack .mylevel.label2 -in .mylevel.middle -side left
    for {set i 1} {$i<=$nfaces} {incr i} {
	set facearray($i) 1
	checkbutton .mylevel.$i -text "$i" -variable facearray($i)
	pack .mylevel.$i -in .mylevel.middle -side left
    }
    
    button .mylevel.refine -text "Refine Tilt"  -command {
	set facelist ""
	GetValue "gui faces" "faces" nfaces
	set n 0
	for {set i 1} {$i<=$nfaces} {incr i} {
            if { $facearray($i) == 1 } {
		    append facelist " $i" 
		    incr n
		}
	}
	if {$n >= 1} {
		destroy .mylevel
		RaiseAbort "Refining Tilt"
		OneCommand "highindex $options(highindex)"
		OneCommand "angmax $options(angmax)"
		OneCommand "tilt $facelist"
	    } else {
		button .mylevel.warn -bg yellow -fg black -activebackground yellow -height 2 \
		    -text "At least one face needed for Tilt"
		pack .mylevel.warn -expand true -fill x -in .mylevel.bottom 
	    }
	    RemoveAbort 
    }
    
    pack .mylevel.refine -in .mylevel.bottom -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help tilt" }
    pack .mylevel.help -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300
}

##########################################
# Crystrot refinement
proc invokeCrystRot {} {
global options

toplevel .mylevel
wm title .mylevel "Crystrot Refinement"
frame .mylevel.top -borderwidth 10
pack .mylevel.top  -side top -fill x
frame .mylevel.bottom -borderwidth 10
pack .mylevel.bottom  -side top -fill x

entry .mylevel.phistart -width 10 -relief sunken -textvariable options(phistart)
label .mylevel.1 -text "Phi start"
entry .mylevel.phistop -width 10 -relief sunken -textvariable options(phistop)
label .mylevel.2 -text "Phi stop"
entry .mylevel.delphi -width 10 -relief sunken -textvariable options(delphi)
label .mylevel.3 -text "Rotation Increment"
entry .mylevel.angmin -width 10 -relief sunken -textvariable options(angmin)
label .mylevel.4 -text "Minimum Rotation Angle"

grid .mylevel.phistart -in .mylevel.top     -row 0 -column 0 -sticky w
grid .mylevel.1 -in .mylevel.top            -row 0 -column 1 -sticky w
grid .mylevel.phistop -in .mylevel.top      -row 1 -column 0 -sticky w
grid .mylevel.2 -in .mylevel.top            -row 1 -column 1 -sticky w
grid .mylevel.delphi -in .mylevel.top       -row 2 -column 0 -sticky w
grid .mylevel.3 -in .mylevel.top            -row 2 -column 1 -sticky w
grid .mylevel.angmin -in .mylevel.top       -row 3 -column 0 -sticky w
grid .mylevel.4 -in .mylevel.top            -row 3 -column 1 -sticky w

button .mylevel.refine -text "Refine Crystrot"  -command {
    destroy .mylevel
    RaiseAbort "Refining Crystrot"
    OneCommand "phi $options(phistart) $options(phistop) $options(delphi)"
    OneCommand "angmin $options(angmin)"
    OneCommand "crystrot"
    RemoveAbort 
}

pack .mylevel.refine -in .mylevel.bottom -side left -fill x
button .mylevel.cancel -text Cancel -command { destroy .mylevel }
pack .mylevel.cancel -in .mylevel.bottom -side left -fill x

wm geometry .mylevel +100+300
}

##########################################
# Crystrot Simplex refinement
proc invokeCrystRotSimplex {} {
global options

toplevel .mylevel
wm title .mylevel "Crystrot Simplex Refinement"
frame .mylevel.top -borderwidth 10
pack .mylevel.top  -side top -fill x
frame .mylevel.bottom -borderwidth 10
pack .mylevel.bottom  -side top -fill x

label .mylevel.textAngle -text "Initial angle:"
scale .mylevel.angle  -from {1.0} -to {90.0} -orient horizontal -length 200 \
                     -digits 2 -resolution {1} -variable options(initangle)
.mylevel.angle set $options(initangle)

grid .mylevel.textAngle -in .mylevel.top -row 0 -column 0 -sticky sw
grid .mylevel.angle     -in .mylevel.top -row 0 -column 1 -sticky w

button .mylevel.refine -text "Refine Crystrot Simplex"  -command {
    destroy .mylevel
    RaiseAbort "Refining Crystrot Simplex"
    OneCommand "initangle $options(initangle)"
    OneCommand "crystrotsimplex"
    RemoveAbort 
}

pack .mylevel.refine -in .mylevel.bottom -side left -fill x
button .mylevel.cancel -text Cancel -command { destroy .mylevel }
pack .mylevel.cancel -in .mylevel.bottom -side left -fill x

wm geometry .mylevel +100+300
}

##########################################
# Build Model
proc BuildModel {title command maxstring} {
    global globalcommand globaltitle
    set globalcommand $command
    set globaltitle $title
    global options
    global modeltype
    set modeltype "1"

    toplevel .mylevel
    wm title .mylevel "Build $globaltitle"
    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth  10
    pack .mylevel.bottom  -side top -fill x

    entry .mylevel.initdistance -width 10 -relief sunken -textvariable options(initdistance)
    label .mylevel.label -text "Initial Distance"

    pack .mylevel.label -side left -in .mylevel.top 
    pack .mylevel.initdistance  -side left -in .mylevel.top

    label .mylevel.label2 -text "model number"
    GetValue "gui $maxstring" "$maxstring" maxmodel
    for {set modelnr 1} {$modelnr<=$maxmodel} {incr modelnr} {
	radiobutton .mylevel.$modelnr -text "$modelnr" -variable modeltype -value "$modelnr"
    }
    pack .mylevel.label2 -in .mylevel.middle -side left
    for {set modelnr 1} {$modelnr<=$maxmodel} {incr modelnr} {
	pack .mylevel.$modelnr -in .mylevel.middle -side left
    }

    button .mylevel.build -text "Build $globaltitle"  -command {
	destroy .mylevel
	OneCommand "initdistance $options(initdistance)"
	OneCommand "$globalcommand $modeltype"
    }
    button .mylevel.cancel -text Cancel -command { destroy .mylevel } 
    pack .mylevel.build -in .mylevel.bottom -side left -fill x
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300
}


##########################################
# Build and Refine Best Model (block cube needle plate sphere)
proc BestModel {title command} {
    global globaltitle globalcommand
    set globaltitle $title
    set globalcommand $command
    global options
# radiobutton relies on 'global' variables
    global refinemode
    set refinemode "d"

    toplevel .mylevel
    wm title .mylevel "Best $globaltitle"
    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth  10
    pack .mylevel.bottom  -side top -fill x

    entry .mylevel.initdistance -width 10 -relief sunken -textvariable options(initdistance)
    label .mylevel.1 -text "Initial Distance"

    label .mylevel.textExtreme   -text Extreme
    label .mylevel.textTolerance -text Tolerance
    scale .mylevel.extreme       -from {0.1} -to {1.0} -orient horizontal -length 200 \
	    -digits 2 -resolution {0.1} -variable options(extreme)
    scale .mylevel.tolerance     -from {0.01} -to {0.1} -orient horizontal -length 200 \
	    -digits 2 -resolution {0.01} -variable options(tol)
    
    .mylevel.extreme set $options(extreme)
    .mylevel.tolerance set $options(tol)
    
    
    grid .mylevel.1             -in .mylevel.top -row 0 -column 0 -sticky sw
    grid .mylevel.initdistance  -in .mylevel.top -row 0 -column 1 -sticky we
    grid .mylevel.textExtreme   -in .mylevel.top -row 1 -column 0 -sticky sw
    grid .mylevel.extreme       -in .mylevel.top -row 1 -column 1 -sticky we
    grid .mylevel.textTolerance -in .mylevel.top -row 2 -column 0 -sticky sw
    grid .mylevel.tolerance     -in .mylevel.top -row 2 -column 1 -sticky we

    label .mylevel.optimisation -text "Optimisation method:"
    radiobutton .mylevel.volume  -text "Volume" -variable refinemode -value "v"
    radiobutton .mylevel.distance -text "Distance Parabolic (separately)"  -variable refinemode -value "d"
    radiobutton .mylevel.amoeba  -text "Distance Simplex (simultaneously)" -variable refinemode -value "a"

#    pack .mylevel.optimisation -side left -fill x -in .mylevel.middle
#    pack .mylevel.volume -side left -fill x -in .mylevel.middle
#    pack .mylevel.distance  -side left -fill x -in .mylevel.middle
#    pack .mylevel.amoeba -side left -fill x -in .mylevel.middle

    grid .mylevel.optimisation  -in .mylevel.middle -row 0 -column 0 -sticky sw
    grid .mylevel.volume        -in .mylevel.middle -row 0 -column 1 -sticky sw
    grid .mylevel.distance      -in .mylevel.middle -row 1 -column 1 -sticky sw
    grid .mylevel.amoeba        -in .mylevel.middle -row 2 -column 1 -sticky sw

    button .mylevel.build -text "Run Best $globaltitle"  -command {
	destroy .mylevel
	RaiseAbort "Searching Best $globaltitle"
	OneCommand "tol $options(tol)"
	OneCommand "extreme $options(extreme)"
	OneCommand "initdistance $options(initdistance)"
	OneCommand "$globalcommand $refinemode"
	RemoveAbort
    }
    button .mylevel.cancel -text Cancel -command { destroy .mylevel } 
    pack .mylevel.build -in .mylevel.bottom -side left -fill x
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300
}

##########################################
# Execute Command

proc execCommand {} {
toplevel .ec
wm title .ec "Execute Command"

button .ec.quit -text Quit -command {destroy .ec}
button .ec.run -text "Run it" -command {
    destroy .ec
    RaiseAbort "Executing $runit"
    OneCommand $runit
    RemoveAbort 
}

pack .ec.quit .ec.run -side right

label .ec.l -text "Command:" -padx 0
entry .ec.cmd -width 30 -relief sunken -textvariable runit
pack .ec.l -side left
pack .ec.cmd -side left -fill x -expand true

bind .ec.cmd <Return>  {
    destroy .ec
    RaiseAbort "Executing $runit"
    OneCommand $runit
    RemoveAbort 
}

focus .ec.cmd
}

##########################################
# Help About

proc helpAbout {} {
tk_messageBox -type ok -message \
"EUHEDRAL

A.M.M. Schreurs, M. Lutz
Utrecht University
The Netherlands
copyright 2001

a.m.m.schreurs@uu.nl
m.lutz@uu.nl"
}

#MAIN
#
# Set up Frames
#


frame .topFrame   -borderwidth 1 -relief raised
set log [text .topFrame.log -width 80 -height 10 -borderwidth 2 -relief \
         raised -setgrid true -yscrollcommand {.topFrame.scroll set}]
scrollbar .topFrame.scroll -command {.topFrame.log yview}

pack .topFrame.scroll -side right -fill y
pack .topFrame.log -side left -fill both -expand true
pack .topFrame   -side top -fill both -expand true

frame .bottomFrame    -borderwidth 1 -relief raised
pack .bottomFrame    -side top -fill x

frame .messageFrame -borderwidth 1 -relief raised
pack .messageFrame    -side top -fill x

#
# Start Euhedral
#

set DoEcho 1
# Initialize Options

set options(pg) "?"
set options(hklread) 1
set options(filterrun) 1
set filename(ins)  ""

# Open Pipe
set ::env(GFORTRAN_UNBUFFERED_ALL) yes
set f [open |euhedral r+]
fconfigure $f -blocking 0

# fileevent $f readable Log1 
LogUntilPrompt
GetValue "\\alias process_name" "PROCESS_NAME=>" euhedralpid
GetStatus
PrintOptions

InfoMessage "first: select ins file"

# Create Menu
menu .mb

# Make it the menu bar for the main Tk window
. configure -menu .mb

# File menu
menu .mb.file -tearoff 0

.mb.file add command -label "Select ins file ..."  \
      -command selectIns
.mb.file add command -label "Select hkl file ..."  \
      -command selectHKL
.mb.file add separator
.mb.file add command -label "Exit" -command {
   LastCommand
   exit  # this will not be reached
}

.mb add cascade -label File -underline 0 -menu .mb.file

# Edit menu
menu .mb.edit -tearoff 0
.mb.edit add command -label "Edit ins file ..." -command {
    if { $filename(ins) != "" } {
	OneCommand "edit"
    }
}
.mb.edit add separator
.mb.edit add command -label "Buildblock"    -command {BuildModel "Block" "buildblock" "blocks"}
.mb.edit add command -label "Buildcube"    -command {BuildModel "Cube" "buildcube" "cubes"}
.mb.edit add command -label "Buildneedle"  -command {BuildModel "Needle" "buildneedle" "needles"}
.mb.edit add command -label "Buildplate"   -command {BuildModel "Plate" "buildplate" "plates"}
.mb.edit add command -label "Buildsphere"  -command {BuildModel "Sphere" "buildsphere" "spheres"}
.mb.edit add separator
.mb.edit add command -label "Bestblock"     -command {BestModel "Block" "bestblock"}
.mb.edit add command -label "Bestcube"     -command {BestModel "Cube" "bestcube"}
.mb.edit add command -label "Bestneedle"   -command {BestModel "Needle" "bestneedle"}
.mb.edit add command -label "Bestplate"    -command {BestModel "Plate" "bestplate"}
.mb.edit add command -label "Bestsphere"   -command {BestModel "Sphere" "bestsphere"}
.mb.edit add separator
.mb.edit add command -label "Add Face xyz" -command {AddFaceXYZ}
.mb.edit add command -label "Add Face hkl" -command {AddFaceHKL}
.mb.edit add command -label "Add Face Sum" -command {AddFaceSum}
.mb.edit add command -label "Delete Face" -command {DeleteFace}
.mb.edit add command -label "Replace Faces" -command {ReplaceFace}
.mb.edit add separator
.mb.edit add command -label "Reduce Indices" -command {OneCommand "reduce"}

.mb add cascade -label Edit -underline 0 -menu .mb.edit

# Options menu
menu .mb.options -tearoff 0
.mb.options add command -label "Select point group" -command selectPg
.mb.options add command -label "Correction method (ABST/ABSG)" -command { Toggle "correction method" correction correction ABST ABSG }
.mb.options add command -label "Centric crystal shape (on/off)" -command { OnOff "centric crystal shape" "centric" "centric" }
.mb.options add command -label "Plot Cartesian Axes(on/off)" -command { OnOff "Plot Cartesian Axes" "plotlab" "plotlab" }
.mb.options add command -label "Plot Crystal Axes (on/off)" -command { OnOff "Plot Crystal Axes" "plotaxes" "plotaxes" }
.mb.options add command -label "Label Vertices (on/off)" -command { OnOff "Label Vertices" "plotcorner" "plotcorner" }
.mb.options add command -label "Vertical View (on/off)" -command { OnOff "vertical" "vertical" "vertical" }
.mb add cascade -label Options -underline 0 -menu .mb.options

# View menu
menu .mb.view -tearoff 0
.mb.view add command -label "View Perp" -command {ViewPerp}
.mb.view add command -label "View Dir" -command {ViewDir}
.mb.view add command -label "View XYZ" -command {ViewXYZ}
.mb.view add separator
.mb.view add command -label "Platon Xtal" -command {OneCommand "plot"}
.mb add cascade -label View -underline 0 -menu .mb.view
.mb.view add separator
.mb.view add command -label "Status" -command {OneCommand "status"}
.mb.view add command -label "Debug Status" -command {PrintOptions}
.mb.view add command -label "Show" -command {OneCommand "show"}

# Command menu
menu .mb.command -tearoff 0
.mb.command add command -label "Execute Command" -command execCommand
.mb.command add separator
.mb.command add command -label "OneRot" -command ExecOneRot
.mb.command add command -label "PartialRot" -command "OneCommand partialrot"
.mb.command add separator
.mb.command add command -label "Test crystal shape" -command "OneCommand test"
.mb add cascade -label Command -underline 0 -menu .mb.command


# Help menu
menu .mb.help -tearoff 0

.mb.help add command -label "Available Commands" -command {
   OneCommand "?"}
.mb.help add command -label "Manual" -command {
   OneCommand "help"}
.mb.help add separator
.mb.help add command -label "About" -command helpAbout

.mb add cascade -label Help -underline 0 -menu .mb.help


# Button Filter
button .filterButton -text Filter -command filter
pack .filterButton -in .bottomFrame -side left -fill x -expand yes

# Button Volume
button .volumeButton -text Volume -command invokeVolume
pack .volumeButton -in .bottomFrame -side left -fill x -expand yes

# Button Distance
button .distanceButton -text Distance -command invokeDistance
pack .distanceButton -in .bottomFrame -side left -fill x -expand yes

# Button Crystrot
button .crystrotButton -text Crystrot  -command invokeCrystRot
pack .crystrotButton -in .bottomFrame -side left -fill x -expand yes

# Button CrystrotSimplex
button .crystrotSimplexButton -text "Crystrot Simplex"  -command invokeCrystRotSimplex
pack .crystrotSimplexButton -in .bottomFrame -side left -fill x -expand yes

# Button Tilt
button .tiltButton -text Tilt -command invokeTilt
pack .tiltButton -in .bottomFrame -side left -fill x -expand yes


proc ViewPerp {} {
    global options
    global facenr 
    set facenr "0"

    toplevel .mylevel
    wm title .mylevel "View Perp"
    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x
    GetValue "gui faces" "faces" nfaces
    label .mylevel.label2 -text "face number"
    pack .mylevel.label2 -in .mylevel.top -side left
    for {set i 1} {$i<=$nfaces} {incr i} {
	radiobutton .mylevel.$i -text "$i" -variable facenr -value "$i"
	pack .mylevel.$i -in .mylevel.top -side left
    }

    radiobutton .mylevel.ver1 -text "vertical on" -variable options(vertical) -value "on"
    pack .mylevel.ver1 -in .mylevel.middle -side left
    radiobutton .mylevel.ver2 -text "vertical off" -variable options(vertical) -value "off"
    pack .mylevel.ver2 -in .mylevel.middle -side left

    button .mylevel.doit -text "View perp"  -command {
	destroy .mylevel
	OneCommand "vertical $options(vertical)"
	OneCommand "viewperp $facenr"
    }

    pack .mylevel.doit -in .mylevel.bottom -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help viewperp" }
    pack .mylevel.help -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300
}

proc ViewDir {} {

    toplevel .mylevel
    wm title .mylevel "View Dir"

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x
    
    entry .mylevel.h -width 10 -relief sunken -textvariable hindex
    label .mylevel.1 -text "h"
    entry .mylevel.k -width 10 -relief sunken -textvariable kindex
    label .mylevel.2 -text "k"
    entry .mylevel.l -width 10 -relief sunken -textvariable lindex
    label .mylevel.3 -text "l"
    grid .mylevel.h -in .mylevel.top    -row 0 -column 0 -sticky w
    grid .mylevel.1 -in .mylevel.top    -row 0 -column 1 -sticky w
    grid .mylevel.k -in .mylevel.top    -row 1 -column 0 -sticky w
    grid .mylevel.2 -in .mylevel.top    -row 1 -column 1 -sticky w
    grid .mylevel.l -in .mylevel.top    -row 2 -column 0 -sticky w
    grid .mylevel.3 -in .mylevel.top    -row 2 -column 1 -sticky w

    radiobutton .mylevel.ver1 -text "vertical on" -variable options(vertical) -value "on"
    pack .mylevel.ver1 -in .mylevel.middle -side left
    radiobutton .mylevel.ver2 -text "vertical off" -variable options(vertical) -value "off"
    pack .mylevel.ver2 -in .mylevel.middle -side left

    button .mylevel.doit -text "View dir"  -command {
	if { $hindex != "" && $kindex != "" && $lindex != "" } {
	    destroy .mylevel
	    OneCommand "vertical $options(vertical)"
	    OneCommand "viewdir $hindex $kindex $lindex"
	} else {
	}
    }

    pack .mylevel.doit -in .mylevel.bottom -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    button .mylevel.help -text Help -command { OneCommand "help viewdir" }
    pack .mylevel.help -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300

}
proc ViewXYZ {} {

    toplevel .mylevel
    wm title .mylevel "View xyz"

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x
    
    entry .mylevel.x -width 10 -relief sunken -textvariable x
    label .mylevel.1 -text "x"
    entry .mylevel.y -width 10 -relief sunken -textvariable y
    label .mylevel.2 -text "y"
    entry .mylevel.z -width 10 -relief sunken -textvariable z
    label .mylevel.3 -text "z"
    grid .mylevel.x -in .mylevel.top    -row 0 -column 0 -sticky w
    grid .mylevel.1 -in .mylevel.top    -row 0 -column 1 -sticky w
    grid .mylevel.y -in .mylevel.top    -row 1 -column 0 -sticky w
    grid .mylevel.2 -in .mylevel.top    -row 1 -column 1 -sticky w
    grid .mylevel.z -in .mylevel.top    -row 2 -column 0 -sticky w
    grid .mylevel.3 -in .mylevel.top    -row 2 -column 1 -sticky w

    radiobutton .mylevel.ver1 -text "vertical on" -variable options(vertical) -value "on"
    pack .mylevel.ver1 -in .mylevel.middle -side left
    radiobutton .mylevel.ver2 -text "vertical off" -variable options(vertical) -value "off"
    pack .mylevel.ver2 -in .mylevel.middle -side left

    button .mylevel.doit -text "View xyz"  -command {
	if { $x != "" && $y != "" && $z != "" } {
	    destroy .mylevel
	    OneCommand "vertical $options(vertical)"
	    OneCommand "viewxyz $x $y $z"
	}
    }

    pack .mylevel.doit -in .mylevel.bottom -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help viewxyz" }
    pack .mylevel.help -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300

}

proc OnOff {title varname comname} {
    Toggle $title $varname $comname on off
}


proc Toggle {title varname comname valon valoff} {
    global options globalcom globalvar

    set globalcom $comname
    set globalvar $varname

    GetStatus

    toplevel .mylevel
    wm title .mylevel "Toggle $title"

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x

    radiobutton .mylevel.1 -text "$title $valon" -variable options($varname) -value "$valon"
    pack .mylevel.1 -in .mylevel.top -side left
    radiobutton .mylevel.2 -text "$title $valoff" -variable options($varname) -value "$valoff"
    pack .mylevel.2 -in .mylevel.top -side left
    
    button .mylevel.doit -text "Ok" -command {
	destroy .mylevel
	OneCommand "$globalcom $options($globalvar)"
    }
    pack .mylevel.doit -in .mylevel.bottom -side left -fill x
    button .mylevel.help -text "Help" -command {OneCommand "help $globalcom"}
    pack .mylevel.help -in .mylevel.bottom -side left -fill x
    wm geometry .mylevel +100+300
}

proc AddFaceXYZ {} {
    global options
    toplevel .mylevel
    wm title .mylevel "Add Face xyz"

    set dist $options(initdistance)

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x
    
    entry .mylevel.x -width 10 -relief sunken -textvariable x
    label .mylevel.1 -text "x"
    entry .mylevel.y -width 10 -relief sunken -textvariable y
    label .mylevel.2 -text "y"
    entry .mylevel.z -width 10 -relief sunken -textvariable z
    label .mylevel.3 -text "z"
    entry .mylevel.dist -width 10 -relief sunken -textvariable dist
    label .mylevel.4 -text "distance"
    grid .mylevel.x -in .mylevel.top    -row 0 -column 0 -sticky w
    grid .mylevel.1 -in .mylevel.top    -row 0 -column 1 -sticky w
    grid .mylevel.y -in .mylevel.top    -row 1 -column 0 -sticky w
    grid .mylevel.2 -in .mylevel.top    -row 1 -column 1 -sticky w
    grid .mylevel.z -in .mylevel.top    -row 2 -column 0 -sticky w
    grid .mylevel.3 -in .mylevel.top    -row 2 -column 1 -sticky w
    grid .mylevel.dist -in .mylevel.top -row 3 -column 0 -sticky w
    grid .mylevel.4 -in .mylevel.top    -row 3 -column 1 -sticky w
    
    button .mylevel.doit -text "Add face"  -command {
	if { $x != "" && $y != "" && $z != "" && $dist != "" } {
	    destroy .mylevel
	    OneCommand "addface $x $y $z $dist"
	}
    }

    pack .mylevel.doit -in .mylevel.bottom -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help addface" }
    pack .mylevel.help -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300

}
proc AddFaceHKL {} {
    global options
    toplevel .mylevel
    wm title .mylevel "Add Face hkl"

    set dist $options(initdistance)

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x
    
    entry .mylevel.x -width 10 -relief sunken -textvariable h
    label .mylevel.1 -text "h"
    entry .mylevel.y -width 10 -relief sunken -textvariable k
    label .mylevel.2 -text "k"
    entry .mylevel.z -width 10 -relief sunken -textvariable l
    label .mylevel.3 -text "l"
    entry .mylevel.dist -width 10 -relief sunken -textvariable dist
    label .mylevel.4 -text "distance"
    grid .mylevel.x -in .mylevel.top    -row 0 -column 0 -sticky w
    grid .mylevel.1 -in .mylevel.top    -row 0 -column 1 -sticky w
    grid .mylevel.y -in .mylevel.top    -row 1 -column 0 -sticky w
    grid .mylevel.2 -in .mylevel.top    -row 1 -column 1 -sticky w
    grid .mylevel.z -in .mylevel.top    -row 2 -column 0 -sticky w
    grid .mylevel.3 -in .mylevel.top    -row 2 -column 1 -sticky w
    grid .mylevel.dist -in .mylevel.top -row 3 -column 0 -sticky w
    grid .mylevel.4 -in .mylevel.top    -row 3 -column 1 -sticky w
    
    button .mylevel.doit -text "Add face"  -command {
	if { $h != "" && $k != "" && $l != "" && $dist != "" } {
	    destroy .mylevel
	    OneCommand "addfacehkl $h $k $l $dist"
	}
    }

    pack .mylevel.doit -in .mylevel.bottom -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.bottom -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help addfacehkl" }
    pack .mylevel.help -in .mylevel.bottom -side left -fill x
    
    wm geometry .mylevel +100+300

}

proc AddFaceSum {} {
    global options
    global facearray

    toplevel .mylevel
    wm title .mylevel "Add Face Sum"

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x

    GetValue "gui faces" "faces" nfaces
    label .mylevel.label2 -text "face number"
    pack .mylevel.label2 -in .mylevel.top -side left

    for {set i 1} {$i<=$nfaces} {incr i} {
        set facearray($i) 0
	checkbutton .mylevel.$i -text "$i" -variable facearray($i)
	pack .mylevel.$i -in .mylevel.top -side left
    }

    button .mylevel.doit -text "Add Face Sum"  -command {
	set facelist ""
	GetValue "gui faces" "faces" nfaces
	set n 0
	for {set i 1} {$i<=$nfaces} {incr i} {
          if { $facearray($i) == 1 } {
	      append facelist " $i" 
	      incr n
	  }
	}
        if {$n >= 2} {
	    destroy .mylevel
	    OneCommand "addfacesum $facelist"
	    
        } else {
	    button .mylevel.warn -bg yellow -fg black -activebackground yellow -height 2 \
               -text "At least two faces needed for AddFaceSum"
	    pack .mylevel.warn -expand true -fill x -in .mylevel.bottom 
	}

    }

    pack .mylevel.doit -in .mylevel.middle -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.middle -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help addfacesum" }
    pack .mylevel.help -in .mylevel.middle -side left -fill x

    
    wm geometry .mylevel +100+300

}
proc DeleteFace {} {
    global options
    global facearray

    toplevel .mylevel
    wm title .mylevel "Delete Face(s)"

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x

    GetValue "gui faces" "faces" nfaces
    label .mylevel.label2 -text "face number"
    pack .mylevel.label2 -in .mylevel.top -side left

    for {set i 1} {$i<=$nfaces} {incr i} {
        set facearray($i) 0
	checkbutton .mylevel.$i -text "$i" -variable facearray($i)
	pack .mylevel.$i -in .mylevel.top -side left
    }

    button .mylevel.doit -text "Delete Faces"  -command {
	set facelist ""
	GetValue "gui faces" "faces" nfaces
	set n 0
	for {set i 1} {$i<=$nfaces} {incr i} {
          if { $facearray($i) == 1 } {
	      append facelist " $i" 
	      incr n
	  }
	}
        if {$n >= 1} {
	    destroy .mylevel
	    OneCommand "deleteface $facelist"
	    
        } else {
	    button .mylevel.warn -bg yellow -fg black -activebackground yellow -height 2 \
               -text "At least one face needed for DeleteFace"
	    pack .mylevel.warn -expand true -fill x -in .mylevel.bottom 
	}

    }

    pack .mylevel.doit -in .mylevel.middle -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.middle -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help deleteface" }
    pack .mylevel.help -in .mylevel.middle -side left -fill x

    
    wm geometry .mylevel +100+300

}

proc ReplaceFace {} {
    global options
    global facearray

    toplevel .mylevel
    wm title .mylevel "Replace Face"

    frame .mylevel.top -borderwidth 10
    pack .mylevel.top  -side top -fill x
    frame .mylevel.middle -borderwidth 10
    pack .mylevel.middle  -side top -fill x
    frame .mylevel.bottom -borderwidth 10
    pack .mylevel.bottom  -side top -fill x

    GetValue "gui faces" "faces" nfaces
    label .mylevel.label2 -text "face number"
    pack .mylevel.label2 -in .mylevel.top -side left

    for {set i 1} {$i<=$nfaces} {incr i} {
        set facearray($i) 0
	checkbutton .mylevel.$i -text "$i" -variable facearray($i)
	pack .mylevel.$i -in .mylevel.top -side left
    }

    button .mylevel.doit -text "Replace Face"  -command {
	set facelist ""
	GetValue "gui faces" "faces" nfaces
	set n 0
	for {set i 1} {$i<=$nfaces} {incr i} {
          if { $facearray($i) == 1 } {
	      append facelist " $i" 
	      incr n
	  }
	}
        if {$n >= 2} {
	    destroy .mylevel
	    OneCommand "replaceface $facelist"
	    
        } else {
	    button .mylevel.warn -bg yellow -fg black -activebackground yellow -height 2 \
               -text "At least two faces needed for ReplaceFace"
	    pack .mylevel.warn -expand true -fill x -in .mylevel.bottom 
	}

    }

    pack .mylevel.doit -in .mylevel.middle -side left -fill x
    button .mylevel.cancel -text Cancel -command { destroy .mylevel }
    pack .mylevel.cancel -in .mylevel.middle -side left -fill x
    button .mylevel.help -text "Help" -command { OneCommand "help replaceface" }
    pack .mylevel.help -in .mylevel.middle -side left -fill x

    
    wm geometry .mylevel +100+300

}

proc ExecOneRot {} {
    RaiseAbort "OneRot"
    OneCommand "onerot"
    RemoveAbort 
}
